"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const js_string_escape_1 = __importDefault(require("js-string-escape"));
const loader_utils_1 = require("loader-utils");
const resolveLoader_1 = __importDefault(require("./resolveLoader"));
// look through remaining loaders and add options to resize loader
async function getRequireStringWithModifiedResizeLoaderOptions(context, remainingRequest, options, resizeLoaderName, resizeLoaderOptionsGenerator) {
    const { loaders, loaderIndex } = context;
    const resizeLoaderResolvedPath = await resolveLoader_1.default(context, resizeLoaderName);
    const resizeLoader = loaders.find(({ path }, index) => index > loaderIndex && path === resizeLoaderResolvedPath);
    if (resizeLoader === undefined)
        throw new Error(`Can't find ${resizeLoaderName} in the list of loaders after webpack-image-srcset-loader`);
    const resizeLoaderOptions = typeof resizeLoader.options === "string"
        ? loader_utils_1.parseQuery("?" + resizeLoader.options)
        : resizeLoader.options;
    const resizeLoaderRequest = resizeLoader.request;
    const resizeLoaderPath = resizeLoader.path;
    const newResizeLoaderQuery = resizeLoaderPath +
        "?" +
        escapeJsonStringForLoader(JSON.stringify(resizeLoaderOptionsGenerator(options.width, options.scale, resizeLoaderOptions)));
    const newRequestString = remainingRequest.replace(resizeLoaderRequest, newResizeLoaderQuery);
    return js_string_escape_1.default(newRequestString);
}
exports.default = getRequireStringWithModifiedResizeLoaderOptions;
// needed so webpack doesn't mistake "!" for query operator "!"
function escapeJsonStringForLoader(s) {
    return s.replace(/!/g, "\\x21");
}
//# sourceMappingURL=getRequireStringWithModifiedResizeLoaderOptions.js.map