"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.defaultResizeLoaderOptionsGenerator = exports.pitch = exports.raw = void 0;
const schema_utils_1 = require("schema-utils");
const sharp_1 = __importDefault(require("sharp"));
const webpack_loader_util_1 = require("@calvin-l/webpack-loader-util");
const getMaxDensity_1 = __importDefault(require("./helpers/getMaxDensity"));
const getOptionFromSize_1 = __importDefault(require("./helpers/getOptionFromSize"));
const getRequireStringWithModifiedResizeLoaderOptions_1 = __importDefault(require("./helpers/getRequireStringWithModifiedResizeLoaderOptions"));
const validateSizes_1 = __importDefault(require("./helpers/validateSizes"));
const options_json_1 = __importDefault(require("./options.json"));
exports.raw = true;
function pitch(remainingRequest) {
    const callback = this.async();
    const defaultOptions = {
        // @ts-expect-error setting sizes as undefined and let validate throw error if it doesn't exist
        sizes: undefined,
        scaleUp: false,
        resizeLoader: "webpack-image-resize-loader",
        resizeLoaderOptionsGenerator: defaultResizeLoaderOptionsGenerator,
        esModule: true,
    };
    const options = {
        ...defaultOptions,
        ...webpack_loader_util_1.getOptions(this),
    };
    schema_utils_1.validate(options_json_1.default, options, {
        name: "webpack-image-srcset-loader",
        baseDataPath: "options",
    });
    validateSizes_1.default(options.sizes);
    generateVirtualModuleCode(this, remainingRequest, options)
        .then((code) => {
        callback(null, code);
    })
        .catch((error) => {
        callback(error, undefined);
    });
}
exports.pitch = pitch;
function default_1(source) {
    return source;
}
exports.default = default_1;
function defaultResizeLoaderOptionsGenerator(width, scale, existingOptions) {
    return {
        ...existingOptions,
        ...((existingOptions === null || existingOptions === void 0 ? void 0 : existingOptions.fileLoaderOptionsGenerator)
            ? {
                fileLoaderOptionsGenerator: existingOptions.fileLoaderOptionsGenerator.toString(),
            }
            : {}),
        // since we filtered out all the width that are too wide,
        // nothing to worry about there, need this to make sure
        // scales larger than 1x works
        scaleUp: true,
        width,
        scale,
    };
}
exports.defaultResizeLoaderOptionsGenerator = defaultResizeLoaderOptionsGenerator;
/**
 * generate
 *
 * const import_0 = require('-!/Users/Calvin/GitHub/webpack-image-srcset-loader/node_modules/file-loader/dist/cjs.js!/Users/Calvin/GitHub/webpack-image-srcset-loader/node_modules/webpack-image-resize-loader/dist/cjs.js?{"scaleUp":true,"scale":0.5}!/Users/Calvin/GitHub/webpack-image-srcset-loader/test/e2e/fixtures/Macaca_nigra_self-portrait_large.jpg');
 * const import_1 = require('-!/Users/Calvin/GitHub/webpack-image-srcset-loader/node_modules/file-loader/dist/cjs.js!/Users/Calvin/GitHub/webpack-image-srcset-loader/node_modules/webpack-image-resize-loader/dist/cjs.js?{"scaleUp":true,"width":300}!/Users/Calvin/GitHub/webpack-image-srcset-loader/test/e2e/fixtures/Macaca_nigra_self-portrait_large.jpg');
 * const import_2 = require('-!/Users/Calvin/GitHub/webpack-image-srcset-loader/node_modules/file-loader/dist/cjs.js!/Users/Calvin/GitHub/webpack-image-srcset-loader/node_modules/webpack-image-resize-loader/dist/cjs.js?{"scaleUp":true}!/Users/Calvin/GitHub/webpack-image-srcset-loader/test/e2e/fixtures/Macaca_nigra_self-portrait_large.jpg');
 * const import_3 = require('-!/Users/Calvin/GitHub/webpack-image-srcset-loader/node_modules/file-loader/dist/cjs.js!/Users/Calvin/GitHub/webpack-image-srcset-loader/node_modules/webpack-image-resize-loader/dist/cjs.js?{"scaleUp":true,"scale":1}!/Users/Calvin/GitHub/webpack-image-srcset-loader/test/e2e/fixtures/Macaca_nigra_self-portrait_large.jpg');
 *
 * export default `${import_0.default || import_0} 1x, ${import_1.default || import_1} 300w, ${import_2.default || import_2}, ${import_3.default || import_3} 2x`;
 */
async function generateVirtualModuleCode(context, remainingRequest, options) {
    const { fs, resourcePath } = context;
    let importBlock = "";
    let exportBlock = "";
    let importCount = 0;
    let width;
    const sizes = options.sizes;
    const maxDensity = getMaxDensity_1.default(sizes);
    const separator = ", ";
    const tsStart = "${";
    const tsEnd = "}";
    for (const size of sizes) {
        // no need for options.scale or options.width if size === "orignal"
        if (size === "original") {
            const requireString = await getRequireStringWithModifiedResizeLoaderOptions_1.default(context, remainingRequest, {}, options.resizeLoader, options.resizeLoaderOptionsGenerator);
            importBlock += `const import_${importCount} = require('-!${requireString}');\n`;
            exportBlock += `${tsStart}import_${importCount}.default || import_${importCount}${tsEnd}${separator}`;
            importCount++;
            continue;
        }
        const resizeLoaderOption = getOptionFromSize_1.default(size, maxDensity);
        if (!options.scaleUp && resizeLoaderOption.width !== undefined) {
            if (width === undefined) {
                const buffer = fs.readFileSync(resourcePath);
                width = (await sharp_1.default(buffer).metadata()).width;
            }
            if (width !== undefined && resizeLoaderOption.width > width)
                continue;
        }
        const requireString = await getRequireStringWithModifiedResizeLoaderOptions_1.default(context, remainingRequest, resizeLoaderOption, options.resizeLoader, options.resizeLoaderOptionsGenerator);
        importBlock += `const import_${importCount} = require('-!${requireString}');\n`;
        exportBlock += `${tsStart}import_${importCount}.default || import_${importCount}${tsEnd} ${size}${separator}`;
        importCount++;
    }
    exportBlock = exportBlock.substring(0, exportBlock.length - 2);
    const result = `
  ${importBlock}
  ${options.esModule ? "export default `" : "module.exports = `"}${exportBlock}\`;`.trim();
    return result;
}
//# sourceMappingURL=index.js.map